from pathlib import Path

def create_service_file(name: str):
    db_name = name.lower()
    repo_module = f"repositories.{db_name}"
    schema_module = f"schemas.{db_name}"
    service_path = Path.cwd() / "services" / f"{db_name}_service.py"

    class_name = "".join([part.capitalize() for part in db_name.split("_")])
    create_class_name = f"{class_name}Create"
    update_class_name = f"{class_name}Update"
    out_class_name = f"{class_name}Out"

    service_code = f'''
from bson import ObjectId
from fastapi import HTTPException
from typing import List

from {repo_module} import (
    create_{db_name},
    get_{db_name},
    get_{db_name}s,
    update_{db_name},
    delete_{db_name},
)
from {schema_module} import {create_class_name}, {update_class_name}, {out_class_name}


async def add_{db_name}({db_name}_data: {create_class_name}) -> {out_class_name}:
    return await create_{db_name}({db_name}_data)


async def remove_{db_name}({db_name}_id: str):
    if not ObjectId.is_valid({db_name}_id):
        raise HTTPException(status_code=400, detail="Invalid {db_name} ID format")

    filter_dict = {{"_id": ObjectId({db_name}_id)}}
    result = await delete_{db_name}(filter_dict)

    if result.deleted_count == 0:
        raise HTTPException(status_code=404, detail="{class_name} not found")


async def retrieve_{db_name}_by_{db_name}_id({db_name}_id: str) -> {out_class_name}:
    if not ObjectId.is_valid({db_name}_id):
        raise HTTPException(status_code=400, detail="Invalid {db_name} ID format")

    filter_dict = {{"_id": ObjectId({db_name}_id)}}
    result = await get_{db_name}(filter_dict)

    if not result:
        raise HTTPException(status_code=404, detail="{class_name} not found")

    return result


async def retrieve_{db_name}s() -> List[{out_class_name}]:
    return await get_{db_name}s()


async def update_{db_name}_by_id({db_name}_id: str, {db_name}_data: {update_class_name}) -> {out_class_name}:
    if not ObjectId.is_valid({db_name}_id):
        raise HTTPException(status_code=400, detail="Invalid {db_name} ID format")

    filter_dict = {{"_id": ObjectId({db_name}_id)}}
    result = await update_{db_name}(filter_dict, {db_name}_data)

    if not result:
        raise HTTPException(status_code=404, detail="{class_name} not found or update failed")

    return result
'''.strip()

    service_path.parent.mkdir(parents=True, exist_ok=True)
    with open(service_path, "w") as f:
        f.write(service_code)

    print(f"✅ Service for '{db_name}' created at services/{db_name}_services.py")



import bcrypt

def hash_password(password: str|bytes) -> bytes:
    if type(password)==str:
        salt = bcrypt.gensalt()
        hashed = bcrypt.hashpw(password.encode('utf-8'), salt)
        return hashed




def check_password(password: str, hashed: bytes) -> bool:

    return bcrypt.checkpw(password.encode('utf-8'), hashed)



from pathlib import Path

def update_main_routes():
    project_path = Path.cwd()
    api_dir = project_path / "api"
    main_file = project_path / "main.py"

    if not api_dir.exists():
        print("⚠️  Couldn't find 'api/' folder. Are you running this from your project root directory?")
        return

    if not main_file.exists():
        print("❌ 'main.py' not found in project root. Make sure you're in the right directory.")
        return

    route_imports = []
    include_lines = []

    for version_dir in sorted(api_dir.glob("v*")):
        if not version_dir.is_dir():
            continue

        version = version_dir.name  # e.g., "v1"

        for py_file in sorted(version_dir.glob("*.py")):
            if py_file.name == "__init__.py":
                continue

            module_name = py_file.stem  # e.g., 'admin'
            module_path = f"api.{version}.{module_name}"
            router_alias = f"{version}_{module_name}_router"

            # Import line
            route_imports.append(f"from {module_path} import router as {router_alias}")

            # Include line
            prefix = f"/{version}"
            include_lines.append(f"app.include_router({router_alias}, prefix='{prefix}')")

    if not route_imports:
        print("⚠️  No route modules found in version folders.")
        return

    # Prepare block
    start_tag = "# --- auto-routes-start ---"
    end_tag = "# --- auto-routes-end ---"
    new_block = f"{start_tag}\n" + "\n".join(route_imports + [""] + include_lines) + f"\n{end_tag}"

    try:
        main_contents = main_file.read_text()
    except Exception as e:
        print(f"❌ Failed to read main.py: {e}")
        return

    if start_tag in main_contents and end_tag in main_contents:
        updated = (
            main_contents.split(start_tag)[0]
            + new_block
            + main_contents.split(end_tag)[1]
        )
    else:
        updated = main_contents.strip() + "\n\n" + new_block

    try:
        main_file.write_text(updated)
        print(f"✅ main.py updated with routes from: {[f.name for f in version_dir.glob('*.py') if f.name != '__init__.py']}")
    except Exception as e:
        print(f"❌ Failed to write to main.py: {e}")



import jwt
import datetime
from datetime import timezone
from core.database import db
from dotenv import load_dotenv
import os
import asyncio
from bson import ObjectId

load_dotenv()
SECRETID = os.getenv("SECRETID")


async def get_secret_dict()->dict:
    result =await db.secret_keys.find_one({"_id":ObjectId(SECRETID)})
    result.pop('_id')
    return result



async def get_secret_and_header():
    
    import random
    
    secrets = await get_secret_dict()
    
    random_key = random.choice(list(secrets.keys()))
    random_secret = secrets[random_key]
    SECRET_KEYS={random_key:random_secret}
    HEADERS = {"kid":random_key}
    result = {
        "SECRET_KEY":SECRET_KEYS,
        "HEADERS":HEADERS
    }
    
    return result



async def create_jwt_member_token(token):
    secrets = await get_secret_and_header()
    SECRET_KEYS= secrets['SECRET_KEY']
    headers= secrets['HEADERS']
    
    payload = {
        'accessToken': token,
        'role':'member',
        'exp': datetime.datetime.now(timezone.utc) + datetime.timedelta(minutes=15)
    }
    
    
    token = jwt.encode(payload, SECRET_KEYS[headers['kid']], algorithm='HS256', headers=headers)

    return token

async def create_jwt_admin_token(token):
    secrets = await get_secret_and_header()
    SECRET_KEYS= secrets['SECRET_KEY']
    headers= secrets['HEADERS']
    
    payload = {
        'accessToken': token,
        'role':'admin',
        'exp': datetime.datetime.now(timezone.utc) + datetime.timedelta(minutes=15)
    }

    
    token = jwt.encode(payload, SECRET_KEYS[headers['kid']], algorithm='HS256', headers=headers)

    return token



async def decode_jwt_token(token):
    """_summary_

    Args:
        token (_type_): _description_

    Raises:
        Exception: _description_

    Returns:
        if decoded true: {'accessToken': '682c99f395ff4782fbea010f', 'role': 'admin', 'exp': 1747825460}
    """
    SECRET_KEYS = await get_secret_dict()
    # Decode header to extract the `kid`
    unverified_header = jwt.get_unverified_header(token)
    kid = unverified_header['kid']

    # Look up the correct key
    key = SECRET_KEYS.get(kid)

    if not key:
        raise Exception("Unknown key ID")

    # Now decode and verify
    try:
        decoded = jwt.decode(token, key, algorithms=['HS256'])
        return decoded
    except jwt.exceptions.ExpiredSignatureError:
        print("expired token")
        return None
    except jwt.exceptions.InvalidSignatureError:
        print("invalid signature")
        return None
    except jwt.exceptions.DecodeError:
        print("Malformed Token")
        return None

async def decode_jwt_token_without_expiration(token):
    SECRET_KEYS = await get_secret_dict()
    # Decode header to extract the `kid`
    unverified_header = jwt.get_unverified_header(token)
    kid = unverified_header['kid']

    # Look up the correct key
    key = SECRET_KEYS.get(kid)

    if not key:
        raise Exception("Unknown key ID")

    # Now decode and verify
    try:
        decoded = jwt.decode(token, key, algorithms=['HS256'])
        return decoded
    except jwt.exceptions.ExpiredSignatureError:
        print("expired token")
        payload = decoded = jwt.decode(token, key, algorithms=['HS256'],options={"verify_exp": False})
        return payload

    except jwt.exceptions.DecodeError:
        print("Malformed Token")
        return None







from schemas.tokens_schema import refreshTokenOut,accessTokenOut,refreshTokenCreate,accessTokenCreate
from repositories.tokens_repo import add_access_tokens,add_refresh_tokens,get_access_tokens,get_refresh_tokens,add_admin_access_tokens
from security.encrypting_jwt import create_jwt_admin_token,create_jwt_member_token,decode_jwt_token,decode_jwt_token_without_expiration
from bson import errors,ObjectId
from fastapi import HTTPException,status
from security.encrypting_jwt import decode_jwt_token




async def generate_member_access_tokens(userId)->accessTokenOut:
    
    
    try:
        obj_id = ObjectId(userId)
    except errors.InvalidId:
        raise   HTTPException(status_code=status.HTTP_401_UNAUTHORIZED,detail="Invalid User Id")    

    new_access_token = await add_access_tokens(token_data=accessTokenCreate(userId=userId))
    new_access_token.accesstoken = await create_jwt_member_token(token=new_access_token.accesstoken)
    
    return new_access_token



async def generate_admin_access_tokens(userId)->accessTokenOut:
    
    try:
        obj_id = ObjectId(userId)
    except errors.InvalidId:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED,detail="Invalid User Id")    # or raise an error / log it    

    new_access_token = await add_admin_access_tokens(token_data=accessTokenCreate(userId=userId))
    new_access_token.accesstoken = await create_jwt_admin_token(token=new_access_token.accesstoken)
    return new_access_token
    
    
    
async def generate_refresh_tokens(userId,accessToken)->refreshTokenOut:

    try:
        obj_id = ObjectId(userId)
    except errors.InvalidId:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED,detail="Invalid User Id While trying to create refresh token")    # or raise an error / log it    

    accessToken = await decode_jwt_token(accessToken)
    if accessToken==None:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED,detail="Failed to decode the accesstoken while trying to create a refreshtoken")
    try:
        obj_id = ObjectId(accessToken['accessToken'])
    except errors.InvalidId:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED,detail="Invalid Access Id While trying to create refresh token")    # or raise an error / log it    

    new_refresh_token =await add_refresh_tokens(token_data=refreshTokenCreate(userId=userId,previousAccessToken=accessToken['accessToken']))
    return new_refresh_token


async def validate_refreshToken(refreshToken:str):
    try:
        obj_id = ObjectId(refreshToken)
    except errors.InvalidId:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED,detail="Invalid Refresh Id")   # or raise an error / log it    

    refresh_token = await get_refresh_tokens(refreshToken=refreshToken)
    if refresh_token:
        new_refresh_token = await generate_refresh_tokens(userId=refresh_token.userId,accessToken=refresh_token.previousAccessToken)
        return new_refresh_token
    else:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND,detail="Couldn't Find Refresh Id")
    


async def validate_member_accesstoken(accessToken:str):
    decodedAccessToken = await decode_jwt_token(token=accessToken)
    try:
        obj_id = ObjectId(decodedAccessToken['accessToken'])
    except errors.InvalidId:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED,detail="Invalid Access Token")   # or raise an error / log it    
    except TypeError:
        return None

    
    if decodedAccessToken:
        validatedAccessToken= await get_access_tokens(accessToken=decodedAccessToken['accessToken'])
    
        if validatedAccessToken:
            return validatedAccessToken
        else:
            raise HTTPException(status_code=status.HTTP_404_NOT_FOUND,detail="Couldn't Find Access Tokens While Validating Member Access Tokens")
    else:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND,detail="Couldn't Find Refresh Tokens While Validating Member Access Tokens")
    
    
async def validate_admin_accesstoken(accessToken:str):
    
    decodedAccessToken = await decode_jwt_token(token=accessToken)
    try:
        obj_id = ObjectId(decodedAccessToken['accessToken'])
    except errors.InvalidId:
        return None  # or raise an error / log it    

    
    if decodedAccessToken:
        if decodedAccessToken['role']=="admin":
            validatedAccessToken= await get_access_tokens(accessToken=decodedAccessToken['accessToken'])
            if type(validatedAccessToken) == type(accessTokenOut(userId="12",accesstoken="sa")):
                return validatedAccessToken
            elif validatedAccessToken=="None":
                return None
            else:
                return "inactive" 
        else:
            return None  
        
    else:
        return None 
    
    
async def validate_expired_admin_accesstoken(accessToken:str):
    
    decodedAccessToken = await decode_jwt_token_without_expiration(token=accessToken)
    try:
        obj_id = ObjectId(decodedAccessToken['accessToken'])
    except errors.InvalidId:
        return None  # or raise an error / log it    

    
    if decodedAccessToken:
        if decodedAccessToken['role']=="admin":
            validatedAccessToken= await get_access_tokens(accessToken=decodedAccessToken['accessToken'])
            if type(validatedAccessToken) == type(accessTokenOut(userId="12",accesstoken="sa")):
                return validatedAccessToken
            elif validatedAccessToken=="None":
                return None
            else:
                return "inactive" 
        else:
            return None  
        
    else:
        return None 
    
    
    
    
    
    
    
    
async def validate_member_accesstoken_without_expiration(accessToken:str):
    decodedAccessToken = await decode_jwt_token_without_expiration(token=accessToken)
    try:
        obj_id = ObjectId(decodedAccessToken['accessToken'])
    except errors.InvalidId:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED,detail="Invalid Access Id")   # or raise an error / log it    

    
    if decodedAccessToken:
        validatedAccessToken= await get_access_tokens(accessToken=decodedAccessToken['accessToken'])
        if validatedAccessToken:
            return validatedAccessToken
        else:
            raise HTTPException(status_code=status.HTTP_404_NOT_FOUND,detail="Couldn't Find Refresh Id")
    else:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND,detail="Couldn't Find Refresh Id")
    


from schemas.imports import *


class refreshedTokenRequest(BaseModel):
    refreshToken:str
class refreshedToken(BaseModel):
    userId:str
    dateCreated:Optional[str]=datetime.now(timezone.utc).isoformat()
    refreshToken:str
    accessToken:str
    @model_validator(mode='before')
    def set_dates(cls,values):
        now_str = datetime.now(timezone.utc).isoformat()
        values['dateCreated']= now_str
        return values

class accessTokenBase(BaseModel):
    userId:str

    
class accessTokenCreate(accessTokenBase):
    dateCreated:Optional[str]=datetime.now(timezone.utc).isoformat()
    @model_validator(mode='before')
    def set_dates(cls,values):
        now_str = datetime.now(timezone.utc).isoformat()
        values['dateCreated']= now_str
        return values

    
class accessTokenOut(accessTokenBase):
    dateCreated:Optional[str]=datetime.now(timezone.utc).isoformat()
    accesstoken: Optional[str] =None
    @model_validator(mode='before')
    def set_values(cls,values):
        if values is None:
            values = {}
        values['accesstoken']= str(values.get('_id'))
        return values
    
    model_config = {
        'populate_by_name': True,
        'arbitrary_types_allowed': True,
    }
    
    
    

    

class refreshTokenBase(BaseModel):
    userId:str
    previousAccessToken:str

    
class refreshTokenCreate(refreshTokenBase):
    dateCreated:Optional[str]=datetime.now(timezone.utc).isoformat()
    @model_validator(mode='before')
    def set_dates(cls,values):
        now_str = datetime.now(timezone.utc).isoformat()
        values['dateCreated']= now_str
        return values

    
class refreshTokenOut(refreshTokenCreate):
    refreshtoken: Optional[str] =None
    @model_validator(mode='before')
    def set_values(cls,values):
        if values is None:
            values = {}
        values['refreshtoken']= str(values.get('_id'))
        return values
    
    model_config = {
        'populate_by_name': True,
        'arbitrary_types_allowed': True,
    }


class TokenOut(BaseModel):
    userId:str
    accesstoken: Optional[str] =None
    refreshtoken: Optional[str] =None
    dateCreated:Optional[str]=datetime.now(timezone.utc).isoformat()
    @model_validator(mode='before')
    def set_dates(cls,values):
        now_str = datetime.now(timezone.utc).isoformat()
        values['dateCreated']= now_str
        return values    



class refreshTokenRequest(BaseModel):
    refreshToken:str


import shutil
from pathlib import Path

def create_project(project_name):
    source = Path(__file__).parent / "templates" / "project_template"
    target = Path.cwd() / project_name

    if target.exists():
        print("❌ Project already exists.")
        return

    shutil.copytree(source, target)
    print(f"✅ Project created at {target}")


# schemas/response_schema.py

from typing import Generic, TypeVar, Optional
from pydantic import BaseModel
from pydantic.generics import GenericModel

T = TypeVar("T")

class APIResponse(GenericModel, Generic[T]):
    status_code: int
    data: Optional[T]
    detail: str



from pathlib import Path

def create_crud_file(name: str):
    from pathlib import Path

    db_name = name.lower()
    schema_path = Path.cwd() / "schemas" / f"{db_name}.py"
    repo_path = Path.cwd() / "repositories" / f"{db_name}.py"

    if not schema_path.exists():
        print(f"❌ Schema file {schema_path} not found.")
        return

    class_name = "".join([part.capitalize() for part in db_name.split("_")]) + "Base"
    update_class_name = "".join([part.capitalize() for part in db_name.split("_")]) + "Update"
    create_class_name = "".join([part.capitalize() for part in db_name.split("_")]) + "Create"
    out_class_name = "".join([part.capitalize() for part in db_name.split("_")]) + "Out"

    crud_code = f'''
from pymongo import ReturnDocument
from core.database import db
from fastapi import HTTPException,status
from typing import List,Optional
from schemas.{db_name} import {update_class_name}, {create_class_name}, {out_class_name}

async def create_{db_name}({db_name}_data: {create_class_name}) -> {out_class_name}:
    {db_name}_dict = {db_name}_data.model_dump()
    result =await db.{db_name}s.insert_one({db_name}_dict)
    result = await db.{db_name}s.find_one(filter={{"id":result.inserted_id}})
    returnable_result = {out_class_name}(**result)
    return returnable_result

async def get_{db_name}(filter_dict: dict) -> Optional[{out_class_name}]:
    try:
        result = await db.{db_name}s.find_one(filter_dict)

        if result is None:
            return None

        return {out_class_name}(**result)

    except Exception as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"An error occurred while fetching {db_name}: {{str(e)}}"
        )
    
async def get_{db_name}s(filter_dict: dict = {{}}) -> List[{out_class_name}]:
    try:
        if filter_dict is None:
            filter_dict = {{}}

        cursor = db.{db_name}s.find(filter_dict)
        {db_name}_list = []

        async for doc in cursor:
            {db_name}_list.append({out_class_name}(**doc))

        return {db_name}_list

    except Exception as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"An error occurred while fetching {db_name}s: {{str(e)}}"
        )
async def update_{db_name}(filter_dict: dict, {db_name}_data: {update_class_name}) -> {out_class_name}:
    result = await db.{db_name}s.find_one_and_update(
        filter_dict,
        {{"$set": {db_name}_data.model_dump()}},
        return_document=ReturnDocument.AFTER
    )
    returnable_result = {out_class_name}(**result)
    return returnable_result

async def delete_{db_name}(filter_dict: dict):
    return await db.{db_name}s.delete_one(filter_dict)
'''.strip()

    with open(repo_path, "w") as f:
        f.write(crud_code)

    print(f"✅ CRUD for '{db_name}' created in repository/{db_name}.py")



import click
from fasterapi.scaffolder.generate_project import create_project
from fasterapi.scaffolder.generate_crud import create_crud_file
from fasterapi.scaffolder.generate_schema import create_schema_file
from fasterapi.scaffolder.generate_service import create_service_file
from fasterapi.scaffolder.generate_route import create_route_file,get_highest_numbered_api_version,get_latest_modified_api_version
from fasterapi.__version__ import __version__
from fasterapi.scaffolder.mount_routes import update_main_routes
from fasterapi.scaffolder.generate_tokens_repo import create_token_file
import subprocess

@click.group()
@click.version_option(__version__, '--version', '-v', message='FasterAPI version %(version)s')
def cli():
    """⚡ FasterAPI CLI — Scaffold FastAPI apps with ease"""
    pass


@cli.command()
@click.argument("name")
def new(name):
    """Create a new FastAPI project."""
    create_project(name)

@cli.command()
@click.argument("name")
def make_crud(name):
    """Generate CRUD repository functions for a schema."""
    create_crud_file(name)
    
@cli.command()
@click.argument("name")
def make_schema(name):
    """Generate Pydantic classes templates for a schema."""
    create_schema_file(name)
    

@cli.command()
@click.argument("name")
def make_service(name):
    """Generates Python services templates to interact with schema and repository."""
    create_service_file(name)
    

@cli.command()
def mount():
    """Updates the main file with routes from the api/v system."""
    update_main_routes()
    
@cli.command()
def run_d():
    """runs the dev environment using the normal uvicorn way"""
    subprocess.run(["uvicorn", "main:app", "--reload"])

@cli.command()
def update():
    """Upgrades a pip package to the latest version"""
    try:
        subprocess.run(["pip", "install", "nats-fasterapi","--upgrade", ], check=True)
        click.secho(f"✅ Fasterapi is now even faster", fg="green")
    except subprocess.CalledProcessError:
        click.secho(f"❌ Failed to speed up fasterapi'", fg="red", err=True)

@cli.command()
@click.argument("name")
@click.option(
    "--version-mode",
    type=click.Choice(["latest-modified", "highest-number"], case_sensitive=True),
    prompt="Select version mode (latest-modified / highest-number)",
    help="Choose whether to use the latest modified version or the highest numbered version"
)
def make_route(name,version_mode):
    """Generate Route function based on templates and examples in schema and services"""
    if version_mode == "latest-modified":
        version = get_latest_modified_api_version()
    else:
        version = get_highest_numbered_api_version()
        
    print(f"Selected API version: {version}")
    create_route_file(name,version)
    
 
@cli.command()
@click.argument("roles", nargs=-1)
def make_token_repo(roles):
    """Generates token repository based on roles (e.g., admin, user, staff). by default will generate for admin and user role always"""
    if not roles:
        # Default roles if none provided
        roles = ["admin",  "user", ]
        click.secho("⚠️ No roles provided. Using default roles: admin, staff, user, guest-editor", fg="yellow")

    create_token_file(roles)
    click.secho("✅ Token repository generated successfully!", fg="green")


if __name__ == "__main__":
    cli()



from pathlib import Path
from datetime import datetime

def create_schema_file(name: str):
    db_name=name.lower()
    schema_dir = Path.cwd() / "schemas"
    schema_path = schema_dir / f"{db_name}.py"
    schema_dir.mkdir(exist_ok=True)

    if schema_path.exists():
        print(f"⚠️  Schema already exists: schemas/{db_name}.py")
        return

    # Convert snake_case to PascalCase
    class_name = "".join(part.capitalize() for part in db_name.split("_"))

    schema_code = f'''
from schemas.imports import *
from pydantic import Field
import time

class {class_name}Base(BaseModel):
    # Add other fields here 
    pass

class {class_name}Create({class_name}Base):
    # Add other fields here 
    date_created: int = Field(default_factory=lambda: int(time.time()))
    last_updated: int = Field(default_factory=lambda: int(time.time()))

class {class_name}Update(BaseModel):
    # Add other fields here 
    last_updated: int = Field(default_factory=lambda: int(time.time()))

class {class_name}Out({class_name}Base):
    # Add other fields here 
    id: Optional[str] =None
    date_created: Optional[int] = None
    last_updated: Optional[int] = None
    
    @model_validator(mode='before')
    def set_dynamic_values(cls,values):
        values['id']= str(values.get('_id'))
        return values
    class Config:
        orm_mode = True
        populate_by_name = True
        arbitrary_types_allowed = True
        json_encoders = {{
            ObjectId: str
        }}
'''.strip()

    with open(schema_path, "w") as f:
        f.write(schema_code)

    print(f"✅ Schema file created: schemas/{db_name}.py")



import os
import logging
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formataddr
from dotenv import load_dotenv
from email_templates.new_sign_in import generate_new_signin_warning_email_from_template

# Load environment variables
load_dotenv()

# Logging setup
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# Required environment variables
REQUIRED_ENV_VARS = ["EMAIL_USERNAME", "EMAIL_PASSWORD", "EMAIL_HOST", "EMAIL_PORT"]

# Check for missing environment variables
missing_vars = [var for var in REQUIRED_ENV_VARS if not os.getenv(var)]

if missing_vars:
    raise EnvironmentError(
        f"Missing required environment variable(s): {', '.join(missing_vars)}. "
        "Please check your .env file."
    )

# Safe to load now
EMAIL_USERNAME = os.getenv("EMAIL_USERNAME")
EMAIL_PASSWORD = os.getenv("EMAIL_PASSWORD")
EMAIL_HOST = os.getenv("EMAIL_HOST")
EMAIL_PORT = int(os.getenv("EMAIL_PORT"))  # Cast after check

# ------------------- Email Sending Function -------------------

def send_html_email_optimized(
    sender_email: str,
    sender_display_name: str,
    receiver_email: str,
    subject: str,
    html_content: str,
    plain_text_content: str,
    smtp_server: str,
    smtp_port: int,
    smtp_login: str,
    smtp_password: str
):
    """Sends an HTML email with plain-text fallback and a display name."""

    formatted_from_address = formataddr((sender_display_name, sender_email))

    msg = MIMEMultipart("alternative")
    msg["From"] = formatted_from_address
    msg["To"] = receiver_email
    msg["Subject"] = subject

    msg.attach(MIMEText(plain_text_content, "plain"))
    msg.attach(MIMEText(html_content, "html"))

    server = None
    try:
        if smtp_port == 465:
            server = smtplib.SMTP_SSL(smtp_server, smtp_port)
            logger.info(f"Connecting to SMTP server {smtp_server}:{smtp_port} using SSL.")
        elif smtp_port in (587, 25):
            server = smtplib.SMTP(smtp_server, smtp_port)
            server.ehlo()
            server.starttls()
            server.ehlo()
            logger.info(f"Connecting to SMTP server {smtp_server}:{smtp_port} using STARTTLS.")
        else:
            raise ValueError("Unsupported SMTP port. Use 465 or 587.")

        server.login(smtp_login, smtp_password)
        logger.info(f"SMTP login successful for user {smtp_login}.")
        server.sendmail(sender_email, receiver_email, msg.as_string())
        logger.info(f"Email sent to {receiver_email} from {sender_display_name} <{sender_email}>.")

    except smtplib.SMTPAuthenticationError as e:
        logger.error(f"SMTP authentication failed: {e}")
        raise
    except smtplib.SMTPConnectError as e:
        logger.error(f"SMTP connection error: {e}")
        raise
    except smtplib.SMTPException as e:
        logger.error(f"SMTP error: {e}")
        raise
    except Exception as e:
        logger.error(f"Unexpected error: {e}")
        raise
    finally:
        if server:
            server.quit()
            logger.info("SMTP connection closed.")

# ------------------- Public Function -------------------

def send_new_signin_email(receiver_email: str, firstName,lastName,time_data,ip_address,location,extra_data):
    """Sends an OTP email for password change."""
    try:
        html_body = generate_new_signin_warning_email_from_template(
            firstName,lastName,time_data,ip_address,location,extra_data
        )

        plain_text = f"""Hello,

This is an automated message sent to tell {firstName} that there was a new sign in
"""

        send_html_email_optimized(
            sender_email=EMAIL_USERNAME,
            sender_display_name="NAT FROM HOSPITAL",
            receiver_email=receiver_email,
            subject="Reset Your Password",
            html_content=html_body,
            plain_text_content=plain_text,
            smtp_server=EMAIL_HOST,
            smtp_port=EMAIL_PORT,
            smtp_login=EMAIL_USERNAME,
            smtp_password=EMAIL_PASSWORD
        )

    except Exception as e:
        logger.error(f"Failed to send OTP email to {receiver_email}: {e}")
        return 1



import os
import re
import sys
from pathlib import Path
from pydantic import BaseModel
import importlib

def get_latest_modified_api_version(base_dir: str = None) -> str:
    """
    Get the latest modified API version directory (e.g., 'v1', 'v2').
    
    Args:
        base_dir (str): Base directory of the project. Defaults to current directory.
    
    Returns:
        str: Name of the latest modified version directory.
    
    Raises:
        FileNotFoundError: If the API directory or version folders are not found.
    """
    if base_dir is None:
        base_path = os.path.join(os.getcwd(), 'api')
    else:
        base_path = os.path.abspath(os.path.join(base_dir, 'api'))

    if not os.path.exists(base_path):
        raise FileNotFoundError(f"The directory '{base_path}' does not exist.")
    
    subdirs = [
        os.path.join(base_path, d) for d in os.listdir(base_path)
        if os.path.isdir(os.path.join(base_path, d))
    ]
    
    if not subdirs:
        raise FileNotFoundError(f"No version folders found in '{base_path}'.")

    latest_subdir = max(subdirs, key=os.path.getmtime)
    return os.path.basename(latest_subdir)

def get_highest_numbered_api_version(base_dir: str = None) -> str:
    """
    Get the highest numbered API version directory (e.g., 'v2' > 'v1').
    
    Args:
        base_dir (str): Base directory of the project. Defaults to current directory.
    
    Returns:
        str: Name of the highest numbered version directory.
    
    Raises:
        FileNotFoundError: If the API directory or version folders are not found.
    """
    if base_dir is None:
        base_path = os.path.join(os.getcwd(), 'api')
    else:
        base_path = os.path.abspath(os.path.join(base_dir, 'api'))

    if not os.path.exists(base_path):
        raise FileNotFoundError(f"The directory '{base_path}' does not exist.")
    
    version_dirs = [
        d for d in os.listdir(base_path)
        if os.path.isdir(os.path.join(base_path, d)) and re.match(r'^v\d+$', d)
    ]

    if not version_dirs:
        raise FileNotFoundError(f"No version folders like 'v1', 'v2' found in '{base_path}'.")

    return max(version_dirs, key=lambda v: int(v[1:]))

def create_route_file(name: str, version: str = None, base_dir: str = None) -> bool:
    """
    Create a FastAPI route file for a given resource name and API version.
    
    Args:
        name (str): Name of the resource (e.g., 'user' or 'order_item').
        version (str): API version (e.g., 'v1'). If None, uses highest numbered version.
        base_dir (str): Base directory of the project. Defaults to current directory.
    
    Returns:
        bool: True if successful, False otherwise.
    """
    # Set base directory
    base_path = Path(base_dir) if base_dir else Path.cwd()
    
    # Ensure schemas and services are in sys.path
    sys.path.append(str(base_path))
    
    # Determine API version
    if not version:
        try:
            version = get_highest_numbered_api_version(base_dir)
        except FileNotFoundError as e:
            print(f"❌ {e}")
            return False

    db_name = name.lower()
    class_name = "".join(part.capitalize() for part in db_name.split("_"))
    
    # Define file paths
    schema_path = base_path / "schemas" / f"{db_name}.py"
    service_path = base_path / "services" / f"{db_name}_service.py"
    repo_path = base_path / "repositories" / f"{db_name}.py"
    route_path = base_path / "api" / version / f"{db_name}.py"
    
    # Check for required files
    for path, desc in [
        (schema_path, "Schema"),
        (service_path, "Service"),
        (repo_path, "Repository")
    ]:
        if not path.exists():
            print(f"❌ {desc} file {path} not found.")
            return False
    
    # Dynamically import schema to verify models
   


    # Generate route code
    route_code = f"""
from fastapi import APIRouter, HTTPException, Query, status, Path
from typing import List
from schemas.response_schema import APIResponse
from schemas.{db_name} import (
    {class_name}Create,
    {class_name}Out,
    {class_name}Base,
    {class_name}Update,
)
from services.{db_name}_service import (
    add_{db_name},
    remove_{db_name},
    retrieve_{db_name}s,
    retrieve_{db_name}_by_{db_name}_id,
    update_{db_name},
)

router = APIRouter(prefix="/{db_name}s", tags=["{class_name}s"])

@router.get("/", response_model=APIResponse[List[{class_name}Out]])
async def list_{db_name}s():
    items = await retrieve_{db_name}s()
    return APIResponse(status_code=200, data=items, detail="Fetched successfully")

@router.get("/me", response_model=APIResponse[{class_name}Out])
async def get_my_{db_name}s(id: str = Query(..., description="{db_name} ID to fetch specific item")):
    items = await retrieve_{db_name}_by_{db_name}_id(id=id)
    return APIResponse(status_code=200, data=items, detail="{db_name}s items fetched")
""" 

    # Write route file
    try:
        route_path.parent.mkdir(parents=True, exist_ok=True)
        with route_path.open("w") as f:
            f.write(route_code)
         
        print(f"✅ Route file created: {route_path}")
        return True
    except Exception as e:
        print(f"❌ Failed to write route file: {e}")
        return False


from setuptools import setup, find_packages
version = {}
with open("fasterapi/__version__.py") as f:
    exec(f.read(), version)
setup(
    name="nats-fasterapi",
     version=version["__version__"],
    description="A CLI tool to scaffold FastAPI projects with CRUD and schema support",
    author="Nathaniel Uriri",
    author_email="nat@uriri.com.ng",

    packages=find_packages(),
    include_package_data=True,

    install_requires=[
        "click",
        "pymongo",
        "pydantic",
        "fastapi[all]",
        "motor",
        "redis",
        "requests",
        "bcrypt",
        "pyjwt",
        "python-dotenv",
    ],

    entry_points={
        "console_scripts": [
            "fasterapi=fasterapi.cli:cli",
        ],
    },

    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],

    python_requires=">=3.8",
)



from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from fastapi import FastAPI, Request, HTTPException
from fastapi.responses import JSONResponse
from schemas.response_schema import APIResponse

app = FastAPI()

app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],  # Allow all origins
    allow_credentials=True,
    allow_methods=["*"],  # Allow all HTTP methods
    allow_headers=["*"],  # Allow all headers
)
@app.exception_handler(HTTPException)
async def custom_http_exception_handler(request: Request, exc: HTTPException):
    return JSONResponse(
        status_code=exc.status_code,
        content=APIResponse(
            status_code=exc.status_code,
            data=None,
            detail=exc.detail,
        ).dict()
    )
@app.get("/")
def read_root():
    return {"message": "Hello from FasterAPI!"}

@app.get("/health")
async def health_check():
    return {"status": "healthy"}
    